@@ -825,12 +825,17 @@ class KantoData:
             columns=["ID", "voc_app_data" if song_level else "unit_app_data"],
         )
         int_df["voc_check" if song_level else "unit_check"] = False
-
+        ID = self.files.ID
+        self.files = self.files.drop(
+            self.files.columns.intersection(int_df.columns), axis=1
+        )
+        self.files["ID"] = ID
         self.files = (
             self.files.reset_index()
             .merge(int_df, how="left", on="ID")
             .set_index("index")
         )
+
         self.save_to_disk(verbose=self.parameters.verbose)
 
     def open_label_app(
@@ -906,10 +911,10 @@ class KantoData:
         # Check palette validity
         max_labs_data = (
             getattr(self, datatype)
-            .dropna(subset=["auto_class"])["auto_class"]
-            .astype(int)
+            .dropna(subset=["auto_class"])
+            .groupby(["ID"])["auto_class"]
+            .nunique()
             .max()
-            + 1
         )
         max_labs = max_n_labs if max_n_labs > max_labs_data else max_labs_data
         if len(palette) < max_labs:
